@@ -8,8 +8,34 @@ from krankshaft.authn import Authn
 from krankshaft.authz import Authz as AuthzBase
 from krankshaft.resource import DjangoModelResource
 from tests.base import TestCaseNoDB
+import base64
 import json
+import os
 import pytest
+import shutil
+import tempfile
+import unittest
+
+try:
+    from PIL import Image
+except ImportError:
+    Image = None
+
+IMAGE_JPG = base64.decodestring(''.join('''
+/9j/4AAQSkZJRgABAQEAAQABAAD/2wBDAAMCAgICAgMCAgIDAwMDBAYEBAQEBAgGBgUGCQgKCgkI
+CQkKDA8MCgsOCwkJDRENDg8QEBEQCgwSExIQEw8QEBD/2wBDAQMDAwQDBAgEBAgQCwkLEBAQEBAQ
+EBAQEBAQEBAQEBAQEBAQEBAQEBAQEBAQEBAQEBAQEBAQEBAQEBAQEBAQEBD/wAARCAABAAEDASIA
+AhEBAxEB/8QAFQABAQAAAAAAAAAAAAAAAAAAAAn/xAAUEAEAAAAAAAAAAAAAAAAAAAAA/8QAFAEB
+AAAAAAAAAAAAAAAAAAAAAP/EABQRAQAAAAAAAAAAAAAAAAAAAAD/2gAMAwEAAhEDEQA/AKOgA//Z
+'''.splitlines()))
+
+IMAGE_JPG_INVALID = base64.decodestring(''.join('''
+/9j/4AAQSkZJRgABAQEAAQABAAD/2wBDAAMCAgICAgMCAgIDAwMDBAYEBAQEBAgGBgUGCQgKCgkI
+AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
+EBAQEBAQEBAQEBAQEBAQEBAQEBAQEBAQEBAQEBAQEBAQEBAQEBAQEBAQEBD/wAARCAABAAEDASIA
+AhEBAxEB/8QAFQABAQAAAAAAAAAAAAAAAAAAAAn/xAAUEAEAAAAAAAAAAAAAAAAAAAAA/8QAFAEB
+AAAAAAAAAAAAAAAAAAAAAP/EABQRAQAAAAAAAAAAAAAAAAAAAAD/2gAMAwEAAhEDEQA/AKOgA//Z
+'''.splitlines()))
 
 class Authz(AuthzBase):
     def is_authorized_object(self, request, authned, obj):
@@ -72,6 +98,10 @@ class ModelVersioned(models.Model):
         self.version += 1
         return super(ModelVersioned, self).save(*args, **kwargs)
 
+class ModelFiles(models.Model):
+    file = models.FileField(max_length=300, upload_to='files/')
+    image = models.ImageField(max_length=300, upload_to='images/')
+
 @pytest.mark.django_db
 class ResourceTest(TestCaseNoDB):
     def _pre_setup(self):
@@ -150,6 +180,10 @@ class ResourceTest(TestCaseNoDB):
             model = ModelVersioned
             version_field = 'version'
 
+        @api
+        class ModelFilesResource(DjangoModelResource):
+            model = ModelFiles
+
         # only to satisfy coverage (calling load() twice)
         for resource in self.api.registered_resources:
             resource.load()
@@ -248,6 +282,71 @@ class ResourceTest(TestCaseNoDB):
             *ids
         )
 
+    def test_files_file(self):
+        tmpdir = tempfile.mkdtemp()
+        try:
+            with self.settings(MEDIA_ROOT=tmpdir), \
+                 tempfile.NamedTemporaryFile() as tmp:
+                tmp.write('hello world\n')
+                tmp.seek(0)
+                response = self.client.post(
+                    self.api.reverse('modelfiles_list'),
+                    {'file': tmp}
+                )
+
+                assert response.status_code == 200
+                assert json.loads(response.content) == {
+                    'file': 'files/' + os.path.basename(tmp.name),
+                    'file_href': '/media/files/' + os.path.basename(tmp.name),
+                    'id': 1,
+                    'image': '',
+                    'image_href': '',
+                    'resource_uri': '/api/v1/modelfiles/1/',
+                }
+
+                response = self.client.put(
+                    self.api.reverse('modelfiles_single', args=(1,)),
+                    {'file': 'wont change anything'}
+                )
+
+                assert response.status_code == 422
+
+        finally:
+            shutil.rmtree(tmpdir, ignore_errors=True)
+
+    @unittest.skipIf(not Image, 'requires PIL/Pillow')
+    def test_files_image(self):
+        tmpdir = tempfile.mkdtemp()
+        try:
+            with self.settings(MEDIA_ROOT=tmpdir), \
+                 tempfile.NamedTemporaryFile() as tmp:
+                tmp.write(IMAGE_JPG)
+                tmp.seek(0)
+                response = self.client.post(
+                    self.api.reverse('modelfiles_list'),
+                    {'image': tmp}
+                )
+
+                assert response.status_code == 200
+                assert json.loads(response.content) == {
+                    'file': '',
+                    'file_href': '',
+                    'id': 2,
+                    'image': 'images/' + os.path.basename(tmp.name),
+                    'image_href': '/media/images/' + os.path.basename(tmp.name),
+                    'resource_uri': '/api/v1/modelfiles/2/',
+                }
+
+                response = self.client.put(
+                    self.api.reverse('modelfiles_single', args=(2,)),
+                    {'image': 'wont change anything'}
+                )
+
+                assert response.status_code == 422
+
+        finally:
+            shutil.rmtree(tmpdir, ignore_errors=True)
+
     def test_get_list(self):
         ModelForeign.objects.create(id=1, char_indexed='value')
         ModelForeign.objects.create(id=2, char_indexed='value2')
@@ -884,6 +983,38 @@ class ResourceTest(TestCaseNoDB):
             },
         }
 
+    def test_query_issues(self):
+        ModelForeign.objects.create(id=1, char_indexed='value')
+        ModelForeign.objects.create(id=2, char_indexed='value1')
+        ModelForeign.objects.create(id=3, char_indexed='value2')
+        ModelForeign.objects.create(id=4, char_indexed='other')
+        response = self.client.get(
+            self.api.reverse('modelforeign_list')
+            + '?order_by=created'
+        )
+
+        assert response.status_code == 403
+        assert json.loads(response.content) == {
+            'error': 'There are issues with your query',
+            'invalid': [
+                'You are required to use an indexed field in the order_by',
+            ],
+        }
+
+    def test_query_issues_invalid_field(self):
+        response = self.client.get(
+            self.api.reverse('modelforeign_list')
+            + '?notavalidfield=1'
+        )
+
+        assert response.status_code == 403
+        assert json.loads(response.content) == {
+            'error': 'There are issues with your query',
+            'invalid': [
+                "ModelForeign has no field named 'notavalidfield'",
+            ],
+        }
+
     def test_resource_deserialize(self):
         response = self.client.post(
             self.api.reverse('modelmany_list'),
